

class Flow(object):
    """Generated from OpenAPI #/components/schemas/Flow.Flow model

    A high level data plane traffic flow  
    Acts as a container for endpoints, frame size, frame rate, duration and packet headers  

    Args
    ----
    - name (str): Unique name of an object that is the primary key for objects found in arrays
    - enable (Union[True, False]): Enable or disable the flow
    - endpoint (Endpoint): A container for different types of endpoints. The endpoint choice dictates the type of flow
    - packet (list[Header]): The packet is a list of traffic protocol headers. The order of traffic protocol headers assigned to the list is the order they will appear on the wire
    - size (Size): The frame size which overrides the total length of the packet
    - rate (Rate): The rate of packet transmission
    - duration (Duration): A container for different transmit durations.
    """
    def __init__(self, name=None, enable=None, endpoint=None, packet=[], size=None, rate=None, duration=None):
        from abstract_open_traffic_generator.flow import Endpoint
        from abstract_open_traffic_generator.flow import Size
        from abstract_open_traffic_generator.flow import Rate
        from abstract_open_traffic_generator.flow import Duration
        if isinstance(name, (str, type(None))) is True:
            self.name = name
        else:
            raise TypeError('name must be an instance of (str, type(None))')
        if isinstance(enable, (bool, type(None))) is True:
            self.enable = enable
        else:
            raise TypeError('enable must be an instance of (bool, type(None))')
        if isinstance(endpoint, (Endpoint, type(None))) is True:
            self.endpoint = endpoint
        else:
            raise TypeError('endpoint must be an instance of (Endpoint, type(None))')
        if isinstance(packet, (list, type(None))) is True:
            self.packet = packet
        else:
            raise TypeError('packet must be an instance of (list, type(None))')
        if isinstance(size, (Size, type(None))) is True:
            self.size = size
        else:
            raise TypeError('size must be an instance of (Size, type(None))')
        if isinstance(rate, (Rate, type(None))) is True:
            self.rate = rate
        else:
            raise TypeError('rate must be an instance of (Rate, type(None))')
        if isinstance(duration, (Duration, type(None))) is True:
            self.duration = duration
        else:
            raise TypeError('duration must be an instance of (Duration, type(None))')


class Endpoint(object):
    """Generated from OpenAPI #/components/schemas/Flow.Endpoint model

    A container for different types of endpoints  
    The endpoint choice dictates the type of flow  

    Args
    ----
    - choice (Union[PortEndpoint, DeviceEndpoint]): The type of endpoint that the flow will originate from
    """
    _CHOICE_MAP = {
        'PortEndpoint': 'port',
        'DeviceEndpoint': 'device',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow import PortEndpoint
        from abstract_open_traffic_generator.flow import DeviceEndpoint
        if isinstance(choice, (PortEndpoint, DeviceEndpoint)) is False:
            raise TypeError('choice must be of type: PortEndpoint, DeviceEndpoint')
        self.__setattr__('choice', Endpoint._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Endpoint._CHOICE_MAP[type(choice).__name__], choice)


class PortEndpoint(object):
    """Generated from OpenAPI #/components/schemas/Flow.PortEndpoint model

    An endpoint that contains a transmit port and 0..n receive ports  

    Args
    ----
    - tx_port_name (str): The unique name of a port that is the transmit port
    - rx_port_names (list[str]): The unique names of ports that are the intended receive ports
    - tx_patterns (list[PortPattern]): A list of custom patterns that will be applied to the transmit port
    """
    def __init__(self, tx_port_name=None, rx_port_names=[], tx_patterns=[]):
        if isinstance(tx_port_name, (str, type(None))) is True:
            self.tx_port_name = tx_port_name
        else:
            raise TypeError('tx_port_name must be an instance of (str, type(None))')
        if isinstance(rx_port_names, (list, type(None))) is True:
            self.rx_port_names = rx_port_names
        else:
            raise TypeError('rx_port_names must be an instance of (list, type(None))')
        if isinstance(tx_patterns, (list, type(None))) is True:
            self.tx_patterns = tx_patterns
        else:
            raise TypeError('tx_patterns must be an instance of (list, type(None))')


class DeviceEndpoint(object):
    """Generated from OpenAPI #/components/schemas/Flow.DeviceEndpoint model

    An endpoint that contains 1..n emulated transmit devices and 1..n emulated receive devices  

    Args
    ----
    - tx_device_names (list[str]): The unique names of emulated devices that will be transmitting. The devices names can be 1..n of DeviceGroup, Device, Ethernet, Vlan, Ipv4 etc
    - rx_device_names (list[str]): The unique names of emulated devices that will be receiving. The devices names can be 1..n of DeviceGroup, Device, Ethernet, Vlan, Ipv4 etc
    - packet_encap (Union[ethernet, vlan, ipv4, ipv6]): The encapsulation determines what packet headers will be auto generated by the traffic generator and will make use of any learned information such as mac addresses, gateway addresses etc
    - src_dst_mesh (Union[none, one_to_one, many_to_many, full_mesh]): TBD
    - route_host_mesh (Union[one_to_one, full_mesh]): TBD
    - bi_directional (Union[True, False]): TBD
    - allow_self_destined (Union[True, False]): TBD
    """
    def __init__(self, tx_device_names=[], rx_device_names=[], packet_encap=None, src_dst_mesh=None, route_host_mesh=None, bi_directional=None, allow_self_destined=None):
        if isinstance(tx_device_names, (list, type(None))) is True:
            self.tx_device_names = tx_device_names
        else:
            raise TypeError('tx_device_names must be an instance of (list, type(None))')
        if isinstance(rx_device_names, (list, type(None))) is True:
            self.rx_device_names = rx_device_names
        else:
            raise TypeError('rx_device_names must be an instance of (list, type(None))')
        if isinstance(packet_encap, (str, type(None))) is True:
            self.packet_encap = packet_encap
        else:
            raise TypeError('packet_encap must be an instance of (str, type(None))')
        if isinstance(src_dst_mesh, (str, type(None))) is True:
            self.src_dst_mesh = src_dst_mesh
        else:
            raise TypeError('src_dst_mesh must be an instance of (str, type(None))')
        if isinstance(route_host_mesh, (str, type(None))) is True:
            self.route_host_mesh = route_host_mesh
        else:
            raise TypeError('route_host_mesh must be an instance of (str, type(None))')
        if isinstance(bi_directional, (bool, type(None))) is True:
            self.bi_directional = bi_directional
        else:
            raise TypeError('bi_directional must be an instance of (bool, type(None))')
        if isinstance(allow_self_destined, (bool, type(None))) is True:
            self.allow_self_destined = allow_self_destined
        else:
            raise TypeError('allow_self_destined must be an instance of (bool, type(None))')


class PortPattern(object):
    """Generated from OpenAPI #/components/schemas/Flow.PortPattern model

    A pattern that is applied to a test port  
    The name of the pattern will be reflected in the port results  

    Args
    ----
    - name (str): Unique name of an object that is the primary key for objects found in arrays
    - offset (int): The offset from the beginning of the packet
    - pattern (str): The value of the pattern
    - mask (str): The mask value to be applied against the pattern
    """
    def __init__(self, name=None, offset=None, pattern=None, mask=None):
        if isinstance(name, (str, type(None))) is True:
            self.name = name
        else:
            raise TypeError('name must be an instance of (str, type(None))')
        if isinstance(offset, (float, int, type(None))) is True:
            self.offset = offset
        else:
            raise TypeError('offset must be an instance of (float, int, type(None))')
        if isinstance(pattern, (str, type(None))) is True:
            self.pattern = pattern
        else:
            raise TypeError('pattern must be an instance of (str, type(None))')
        if isinstance(mask, (str, type(None))) is True:
            self.mask = mask
        else:
            raise TypeError('mask must be an instance of (str, type(None))')


class Header(object):
    """Generated from OpenAPI #/components/schemas/Flow.Header model

    Container for all traffic packet headers  

    Args
    ----
    - choice (Union[Custom, Ethernet, Vlan, Ipv4, PfcPause]): TBD
    """
    _CHOICE_MAP = {
        'Custom': 'custom',
        'Ethernet': 'ethernet',
        'Vlan': 'vlan',
        'Ipv4': 'ipv4',
        'PfcPause': 'pfcpause',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow import Custom
        from abstract_open_traffic_generator.flow import Ethernet
        from abstract_open_traffic_generator.flow import Vlan
        from abstract_open_traffic_generator.flow import Ipv4
        from abstract_open_traffic_generator.flow import PfcPause
        if isinstance(choice, (Custom, Ethernet, Vlan, Ipv4, PfcPause)) is False:
            raise TypeError('choice must be of type: Custom, Ethernet, Vlan, Ipv4, PfcPause')
        self.__setattr__('choice', Header._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Header._CHOICE_MAP[type(choice).__name__], choice)


class Custom(object):
    """Generated from OpenAPI #/components/schemas/Flow.Custom model

    Custom packet header  

    Args
    ----
    - bytes (str): A custom packet header defined as a string of hex bytes. The string MUST contain valid hex characters. Spaces or colons can be part of the bytes but will be discarded This can be used to create a custom protocol from other inputs such as scapy, wireshark, pcap etc. An example of ethernet/ipv4: '00000000000200000000000108004500001400010000400066e70a0000010a000002'
    - patterns (list[BitPattern]): Modify the bytes with bit based patterns
    """
    def __init__(self, bytes=None, patterns=[]):
        if isinstance(bytes, (str, type(None))) is True:
            self.bytes = bytes
        else:
            raise TypeError('bytes must be an instance of (str, type(None))')
        if isinstance(patterns, (list, type(None))) is True:
            self.patterns = patterns
        else:
            raise TypeError('patterns must be an instance of (list, type(None))')


class BitPattern(object):
    """Generated from OpenAPI #/components/schemas/Flow.BitPattern model

    Container for a bit pattern  

    Args
    ----
    - choice (Union[BitList, BitCounter]): TBD
    """
    _CHOICE_MAP = {
        'BitList': 'bitlist',
        'BitCounter': 'bitcounter',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow import BitList
        from abstract_open_traffic_generator.flow import BitCounter
        if isinstance(choice, (BitList, BitCounter)) is False:
            raise TypeError('choice must be of type: BitList, BitCounter')
        self.__setattr__('choice', BitPattern._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(BitPattern._CHOICE_MAP[type(choice).__name__], choice)


class BitList(object):
    """Generated from OpenAPI #/components/schemas/Flow.BitList model

    A pattern which is a list of values  

    Args
    ----
    - offset (int): Bit offset in the packet at which the pattern will be applied
    - length (int): The number of bits in the packet that the pattern will span
    - count (int): The number of values to generate before repeating
    - values (list[str]): TBD
    """
    def __init__(self, offset=1, length=1, count=1, values=[]):
        if isinstance(offset, (float, int, type(None))) is True:
            self.offset = offset
        else:
            raise TypeError('offset must be an instance of (float, int, type(None))')
        if isinstance(length, (float, int, type(None))) is True:
            self.length = length
        else:
            raise TypeError('length must be an instance of (float, int, type(None))')
        if isinstance(count, (float, int, type(None))) is True:
            self.count = count
        else:
            raise TypeError('count must be an instance of (float, int, type(None))')
        if isinstance(values, (list, type(None))) is True:
            self.values = values
        else:
            raise TypeError('values must be an instance of (list, type(None))')


class BitCounter(object):
    """Generated from OpenAPI #/components/schemas/Flow.BitCounter model

    An incrementing pattern  

    Args
    ----
    - offset (int): Bit offset in the packet at which the pattern will be applied
    - length (int): The number of bits in the packet that the pattern will span
    - count (int): The number of values to generate before repeating A value of 0 means the pattern will count continuously
    - start (str): The starting value of the pattern. If the value is greater than the length it will be truncated
    - step (str): The amount the start value will be incremented by If the value is greater than the length it will be truncated
    """
    def __init__(self, offset=0, length=32, count=1, start='0', step='0'):
        if isinstance(offset, (float, int, type(None))) is True:
            self.offset = offset
        else:
            raise TypeError('offset must be an instance of (float, int, type(None))')
        if isinstance(length, (float, int, type(None))) is True:
            self.length = length
        else:
            raise TypeError('length must be an instance of (float, int, type(None))')
        if isinstance(count, (float, int, type(None))) is True:
            self.count = count
        else:
            raise TypeError('count must be an instance of (float, int, type(None))')
        if isinstance(start, (str, type(None))) is True:
            self.start = start
        else:
            raise TypeError('start must be an instance of (str, type(None))')
        if isinstance(step, (str, type(None))) is True:
            self.step = step
        else:
            raise TypeError('step must be an instance of (str, type(None))')


class Ethernet(object):
    """Generated from OpenAPI #/components/schemas/Flow.Ethernet model

    Ethernet packet header  

    Args
    ----
    - dst (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - src (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - ether_type (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    """
    def __init__(self, dst=None, src=None, ether_type=None):
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(dst, (Pattern, type(None))) is True:
            self.dst = dst
        else:
            raise TypeError('dst must be an instance of (Pattern, type(None))')
        if isinstance(src, (Pattern, type(None))) is True:
            self.src = src
        else:
            raise TypeError('src must be an instance of (Pattern, type(None))')
        if isinstance(ether_type, (Pattern, type(None))) is True:
            self.ether_type = ether_type
        else:
            raise TypeError('ether_type must be an instance of (Pattern, type(None))')


class Pattern(object):
    """Generated from OpenAPI #/components/schemas/Flow.Pattern model

    A container for packet header field patterns  
    Possible patterns are fixed, list, counter, random  

    Args
    ----
    - choice (Union[str, list, Counter, Random]): TBD
    - ingress_result_name (str): A unique name is used to indicate to the system that the field may extend the result row key and create an aggregate result row for every unique ingress value. To have ingress columns appear in flow result rows the flow result request allows for the ingress_result_name value to be specified as part of the request
    """
    _CHOICE_MAP = {
        'str': 'fixed',
        'list': 'list',
        'Counter': 'counter',
        'Random': 'random',
    }
    def __init__(self, choice=None, ingress_result_name=None):
        from abstract_open_traffic_generator.flow import Counter
        from abstract_open_traffic_generator.flow import Random
        if isinstance(choice, (str, list, Counter, Random)) is False:
            raise TypeError('choice must be of type: str, list, Counter, Random')
        self.__setattr__('choice', Pattern._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Pattern._CHOICE_MAP[type(choice).__name__], choice)
        if isinstance(ingress_result_name, (str, type(None))) is True:
            self.ingress_result_name = ingress_result_name
        else:
            raise TypeError('ingress_result_name must be an instance of (str, type(None))')


class Counter(object):
    """Generated from OpenAPI #/components/schemas/Flow.Counter model

    A counter pattern that can increment or decrement  

    Args
    ----
    - start (str): The value at which the pattern will start
    - step (str): The value at which the pattern will increment or decrement by
    - count (Union[float, int]): The number of values in the pattern
    - up (Union[True, False]): Determines whether the pattern will increment (true) or decrement (false)
    """
    def __init__(self, start=None, step=None, count=None, up=True):
        if isinstance(start, (str, type(None))) is True:
            self.start = start
        else:
            raise TypeError('start must be an instance of (str, type(None))')
        if isinstance(step, (str, type(None))) is True:
            self.step = step
        else:
            raise TypeError('step must be an instance of (str, type(None))')
        if isinstance(count, (float, int, type(None))) is True:
            self.count = count
        else:
            raise TypeError('count must be an instance of (float, int, type(None))')
        if isinstance(up, (bool, type(None))) is True:
            self.up = up
        else:
            raise TypeError('up must be an instance of (bool, type(None))')


class Random(object):
    """Generated from OpenAPI #/components/schemas/Flow.Random model

    A repeatable random range pattern  

    Args
    ----
    - min (str): TBD
    - max (str): TBD
    - step (Union[float, int]): TBD
    - seed (str): TBD
    - count (Union[float, int]): TBD
    """
    def __init__(self, min=None, max=None, step=None, seed=None, count=None):
        if isinstance(min, (str, type(None))) is True:
            self.min = min
        else:
            raise TypeError('min must be an instance of (str, type(None))')
        if isinstance(max, (str, type(None))) is True:
            self.max = max
        else:
            raise TypeError('max must be an instance of (str, type(None))')
        if isinstance(step, (float, int, type(None))) is True:
            self.step = step
        else:
            raise TypeError('step must be an instance of (float, int, type(None))')
        if isinstance(seed, (str, type(None))) is True:
            self.seed = seed
        else:
            raise TypeError('seed must be an instance of (str, type(None))')
        if isinstance(count, (float, int, type(None))) is True:
            self.count = count
        else:
            raise TypeError('count must be an instance of (float, int, type(None))')


class Vlan(object):
    """Generated from OpenAPI #/components/schemas/Flow.Vlan model

    Vlan packet header  

    Args
    ----
    - priority (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - cfi (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - id (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - protocol (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    """
    def __init__(self, priority=None, cfi=None, id=None, protocol=None):
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(priority, (Pattern, type(None))) is True:
            self.priority = priority
        else:
            raise TypeError('priority must be an instance of (Pattern, type(None))')
        if isinstance(cfi, (Pattern, type(None))) is True:
            self.cfi = cfi
        else:
            raise TypeError('cfi must be an instance of (Pattern, type(None))')
        if isinstance(id, (Pattern, type(None))) is True:
            self.id = id
        else:
            raise TypeError('id must be an instance of (Pattern, type(None))')
        if isinstance(protocol, (Pattern, type(None))) is True:
            self.protocol = protocol
        else:
            raise TypeError('protocol must be an instance of (Pattern, type(None))')


class Ipv4(object):
    """Generated from OpenAPI #/components/schemas/Flow.Ipv4 model

    Ipv4 packet header  

    Args
    ----
    - priority (Priority): Ipv4 ip priority that can be one of RAW or DSCP
    - src (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - dst (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    """
    def __init__(self, priority=None, src=None, dst=None):
        from abstract_open_traffic_generator.flow_ipv4 import Priority
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(priority, (Priority, type(None))) is True:
            self.priority = priority
        else:
            raise TypeError('priority must be an instance of (Priority, type(None))')
        if isinstance(src, (Pattern, type(None))) is True:
            self.src = src
        else:
            raise TypeError('src must be an instance of (Pattern, type(None))')
        if isinstance(dst, (Pattern, type(None))) is True:
            self.dst = dst
        else:
            raise TypeError('dst must be an instance of (Pattern, type(None))')


class PfcPause(object):
    """Generated from OpenAPI #/components/schemas/Flow.PfcPause model

    IEEE 802.1Qbb PFC Pause packet header  
    - dst: 01:80:C2:00:00:01 48bits - src: 48bits - ether_type: 0x8808 16bits - control_op_code: 0x0101 16bits - class_enable_vector: 16bits - pause_class_0: 0x0000 16bits - pause_class_1: 0x0000 16bits - pause_class_2: 0x0000 16bits - pause_class_3: 0x0000 16bits - pause_class_4: 0x0000 16bits - pause_class_5: 0x0000 16bits - pause_class_6: 0x0000 16bits - pause_class_7: 0x0000 16bits  

    Args
    ----
    - dst (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - src (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - ether_type (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - control_op_code (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - class_enable_vector (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_0 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_1 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_2 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_3 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_4 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_5 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_6 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - pause_class_7 (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    """
    def __init__(self, dst=None, src=None, ether_type=None, control_op_code=None, class_enable_vector=None, pause_class_0=None, pause_class_1=None, pause_class_2=None, pause_class_3=None, pause_class_4=None, pause_class_5=None, pause_class_6=None, pause_class_7=None):
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(dst, (Pattern, type(None))) is True:
            self.dst = dst
        else:
            raise TypeError('dst must be an instance of (Pattern, type(None))')
        if isinstance(src, (Pattern, type(None))) is True:
            self.src = src
        else:
            raise TypeError('src must be an instance of (Pattern, type(None))')
        if isinstance(ether_type, (Pattern, type(None))) is True:
            self.ether_type = ether_type
        else:
            raise TypeError('ether_type must be an instance of (Pattern, type(None))')
        if isinstance(control_op_code, (Pattern, type(None))) is True:
            self.control_op_code = control_op_code
        else:
            raise TypeError('control_op_code must be an instance of (Pattern, type(None))')
        if isinstance(class_enable_vector, (Pattern, type(None))) is True:
            self.class_enable_vector = class_enable_vector
        else:
            raise TypeError('class_enable_vector must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_0, (Pattern, type(None))) is True:
            self.pause_class_0 = pause_class_0
        else:
            raise TypeError('pause_class_0 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_1, (Pattern, type(None))) is True:
            self.pause_class_1 = pause_class_1
        else:
            raise TypeError('pause_class_1 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_2, (Pattern, type(None))) is True:
            self.pause_class_2 = pause_class_2
        else:
            raise TypeError('pause_class_2 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_3, (Pattern, type(None))) is True:
            self.pause_class_3 = pause_class_3
        else:
            raise TypeError('pause_class_3 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_4, (Pattern, type(None))) is True:
            self.pause_class_4 = pause_class_4
        else:
            raise TypeError('pause_class_4 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_5, (Pattern, type(None))) is True:
            self.pause_class_5 = pause_class_5
        else:
            raise TypeError('pause_class_5 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_6, (Pattern, type(None))) is True:
            self.pause_class_6 = pause_class_6
        else:
            raise TypeError('pause_class_6 must be an instance of (Pattern, type(None))')
        if isinstance(pause_class_7, (Pattern, type(None))) is True:
            self.pause_class_7 = pause_class_7
        else:
            raise TypeError('pause_class_7 must be an instance of (Pattern, type(None))')


class Size(object):
    """Generated from OpenAPI #/components/schemas/Flow.Size model

    The frame size which overrides the total length of the packet  

    Args
    ----
    - choice (Union[float, int, SizeIncrement, SizeRandom]): TBD
    """
    _CHOICE_MAP = {
        'float': 'fixed',
        'int': 'fixed',
        'SizeIncrement': 'increment',
        'SizeRandom': 'random',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow import SizeIncrement
        from abstract_open_traffic_generator.flow import SizeRandom
        if isinstance(choice, (float, int, SizeIncrement, SizeRandom)) is False:
            raise TypeError('choice must be of type: float, int, SizeIncrement, SizeRandom')
        self.__setattr__('choice', Size._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Size._CHOICE_MAP[type(choice).__name__], choice)


class SizeIncrement(object):
    """Generated from OpenAPI #/components/schemas/Flow.SizeIncrement model

    Frame size that increments from a starting size to an ending size incrementing by a step size  

    Args
    ----
    - start (int): Starting frame size in bytes
    - end (int): Ending frame size in bytes
    - step (int): Step frame size in bytes
    """
    def __init__(self, start=64, end=1518, step=1):
        if isinstance(start, (float, int, type(None))) is True:
            self.start = start
        else:
            raise TypeError('start must be an instance of (float, int, type(None))')
        if isinstance(end, (float, int, type(None))) is True:
            self.end = end
        else:
            raise TypeError('end must be an instance of (float, int, type(None))')
        if isinstance(step, (float, int, type(None))) is True:
            self.step = step
        else:
            raise TypeError('step must be an instance of (float, int, type(None))')


class SizeRandom(object):
    """Generated from OpenAPI #/components/schemas/Flow.SizeRandom model

    Random frame size from a min value to a max value  

    Args
    ----
    - min (int): TBD
    - max (int): TBD
    """
    def __init__(self, min=64, max=1518):
        if isinstance(min, (float, int, type(None))) is True:
            self.min = min
        else:
            raise TypeError('min must be an instance of (float, int, type(None))')
        if isinstance(max, (float, int, type(None))) is True:
            self.max = max
        else:
            raise TypeError('max must be an instance of (float, int, type(None))')


class Rate(object):
    """Generated from OpenAPI #/components/schemas/Flow.Rate model

    The rate of packet transmission  

    Args
    ----
    - unit (Union[pps, bps, kbps, mbps, gbps, line]): The value is a unit of this
    - value (int): The actual rate
    """
    def __init__(self, unit=None, value=None):
        if isinstance(unit, (str, type(None))) is True:
            self.unit = unit
        else:
            raise TypeError('unit must be an instance of (str, type(None))')
        if isinstance(value, (float, int, type(None))) is True:
            self.value = value
        else:
            raise TypeError('value must be an instance of (float, int, type(None))')


class Duration(object):
    """Generated from OpenAPI #/components/schemas/Flow.Duration model

    A container for different transmit durations  

    Args
    ----
    - choice (Union[Fixed, Burst]): TBD
    """
    _CHOICE_MAP = {
        'Fixed': 'fixed',
        'Burst': 'burst',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow import Fixed
        from abstract_open_traffic_generator.flow import Burst
        if isinstance(choice, (Fixed, Burst)) is False:
            raise TypeError('choice must be of type: Fixed, Burst')
        self.__setattr__('choice', Duration._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Duration._CHOICE_MAP[type(choice).__name__], choice)


class Fixed(object):
    """Generated from OpenAPI #/components/schemas/Flow.Fixed model

    A fixed number of packets will be transmitted after which the flow will stop  
    If the number of packets is set to 0 the flow will not stop  

    Args
    ----
    - packets (int): Stop transmit of the flow after this number of packets. A value of 0 means that the flow will not stop transmitting
    - gap (int): The minimum gap between packets expressed as bytes
    - delay (int): The delay before starting transmission of packets
    - delay_unit (Union[bytes, nanoseconds]): The delay expressed as a number of this value
    """
    def __init__(self, packets=0, gap=None, delay=None, delay_unit=None):
        if isinstance(packets, (float, int, type(None))) is True:
            self.packets = packets
        else:
            raise TypeError('packets must be an instance of (float, int, type(None))')
        if isinstance(gap, (float, int, type(None))) is True:
            self.gap = gap
        else:
            raise TypeError('gap must be an instance of (float, int, type(None))')
        if isinstance(delay, (float, int, type(None))) is True:
            self.delay = delay
        else:
            raise TypeError('delay must be an instance of (float, int, type(None))')
        if isinstance(delay_unit, (str, type(None))) is True:
            self.delay_unit = delay_unit
        else:
            raise TypeError('delay_unit must be an instance of (str, type(None))')


class Burst(object):
    """Generated from OpenAPI #/components/schemas/Flow.Burst model

    A continuous burst of packets that will not automatically stop  

    Args
    ----
    - packets (int): The number of packets transmitted per burst
    - gap (int): The minimum gap between packets expressed as bytes
    - inter_burst_gap (int): The gap between the transmission of each burst. A value of 0 means there is no gap between bursts
    - inter_burst_gap_unit (Union[bytes, nanoseconds]): The inter burst gap expressed as a number of this value
    """
    def __init__(self, packets=None, gap=None, inter_burst_gap=None, inter_burst_gap_unit=None):
        if isinstance(packets, (float, int, type(None))) is True:
            self.packets = packets
        else:
            raise TypeError('packets must be an instance of (float, int, type(None))')
        if isinstance(gap, (float, int, type(None))) is True:
            self.gap = gap
        else:
            raise TypeError('gap must be an instance of (float, int, type(None))')
        if isinstance(inter_burst_gap, (float, int, type(None))) is True:
            self.inter_burst_gap = inter_burst_gap
        else:
            raise TypeError('inter_burst_gap must be an instance of (float, int, type(None))')
        if isinstance(inter_burst_gap_unit, (str, type(None))) is True:
            self.inter_burst_gap_unit = inter_burst_gap_unit
        else:
            raise TypeError('inter_burst_gap_unit must be an instance of (str, type(None))')





class Options(object):
    """Generated from OpenAPI #/components/schemas/Port.Options model

    Common port options that apply to all configured Port.Port objects  

    Args
    ----
    - location_preemption (Union[True, False]): Preempt all the test port locations as defined by the Port.Port.properties.location. If the test ports as defined by their location values are in use and this value is true, the test ports will be preempted
    """
    def __init__(self, location_preemption=False):
        if isinstance(location_preemption, (bool, type(None))) is True:
            self.location_preemption = location_preemption
        else:
            raise TypeError('location_preemption must be an instance of (bool, type(None))')


class Port(object):
    """Generated from OpenAPI #/components/schemas/Port.Port model

    An abstract test port  

    Args
    ----
    - name (str): Unique name of an object that is the primary key for objects found in arrays
    - location (str): The location of a test port. It is the endpoint where packets will emit from. Test port locations can be the following:
        physical appliance with multiple ports
        physical chassis with multiple cards and ports
        local interface
        virtual machine, docker container, kubernetes cluster The test port location format is implementation specific. Use the /results/capabilities API to determine what formats an implementation supports for the location property. Get the configured location state by using the /results/port API
    """
    def __init__(self, name=None, location=None):
        if isinstance(name, (str, type(None))) is True:
            self.name = name
        else:
            raise TypeError('name must be an instance of (str, type(None))')
        if isinstance(location, (str, type(None))) is True:
            self.location = location
        else:
            raise TypeError('location must be an instance of (str, type(None))')





class Priority(object):
    """Generated from OpenAPI #/components/schemas/Flow.Ipv4.Priority model

    Ipv4 ip priority that can be one of RAW or DSCP  

    Args
    ----
    - choice (Union[Dscp, Pattern]): TBD
    """
    
    PRIORITY_RAW = '0'
    
    _CHOICE_MAP = {
        'Dscp': 'dscp',
        'Pattern': 'raw',
    }
    def __init__(self, choice=None):
        from abstract_open_traffic_generator.flow_ipv4 import Dscp
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(choice, (Dscp, Pattern)) is False:
            raise TypeError('choice must be of type: Dscp, Pattern')
        self.__setattr__('choice', Priority._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Priority._CHOICE_MAP[type(choice).__name__], choice)


class Dscp(object):
    """Generated from OpenAPI #/components/schemas/Flow.Ipv4.Dscp model

    Differentiated services code point (DSCP) packet field  
    PHB (per-hop-behavior) value is 6 bits: >=0 PHB <=63  
    ECN (explicit-congestion-notification) value is 2 bits: >=0 ECN <=3  

    Args
    ----
    - phb (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    - ecn (Pattern): A container for packet header field patterns. Possible patterns are fixed, list, counter, random
    """
    
    PHB_DEFAULT = '0'
    PHB_CS1 = '8'
    PHB_CS2 = '16'
    PHB_CS3 = '24'
    PHB_CS4 = '32'
    PHB_CS5 = '40'
    PHB_CS6 = '48'
    PHB_CS7 = '56'
    PHB_AF11 = '10'
    PHB_AF12 = '12'
    PHB_AF13 = '14'
    PHB_AF21 = '18'
    PHB_AF22 = '20'
    PHB_AF23 = '22'
    PHB_AF31 = '26'
    PHB_AF32 = '28'
    PHB_AF33 = '30'
    PHB_AF41 = '24'
    PHB_AF42 = '36'
    PHB_AF43 = '38'
    PHB_EF46 = '46'
    ECN_NON_CAPABLE = '0'
    ECN_CAPABLE_TRANSPORT_0 = '1'
    ECN_CAPABLE_TRANSPORT_1 = '2'
    ECN_CONGESTION_ENCOUNTERED = '3'
    
    def __init__(self, phb=None, ecn=None):
        from abstract_open_traffic_generator.flow import Pattern
        if isinstance(phb, (Pattern, type(None))) is True:
            self.phb = phb
        else:
            raise TypeError('phb must be an instance of (Pattern, type(None))')
        if isinstance(ecn, (Pattern, type(None))) is True:
            self.ecn = ecn
        else:
            raise TypeError('ecn must be an instance of (Pattern, type(None))')





class Layer1(object):
    """Generated from OpenAPI #/components/schemas/Layer1.Layer1 model

    A container for layer1 ports and settings  

    Args
    ----
    - name (str): Unique name of an object that is the primary key for objects found in arrays
    - port_names (list[str]): A list of unique names of a port objects that will share the choice settings.
    - choice (Union[OneHundredGbe, Ethernet]): The type of layer1 characteristics
    """
    _CHOICE_MAP = {
        'OneHundredGbe': 'one_hundred_gbe',
        'Ethernet': 'ethernet',
    }
    def __init__(self, name=None, port_names=[], choice=None):
        from abstract_open_traffic_generator.layer1 import OneHundredGbe
        from abstract_open_traffic_generator.layer1 import Ethernet
        if isinstance(choice, (OneHundredGbe, Ethernet)) is False:
            raise TypeError('choice must be of type: OneHundredGbe, Ethernet')
        self.__setattr__('choice', Layer1._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(Layer1._CHOICE_MAP[type(choice).__name__], choice)
        if isinstance(name, (str, type(None))) is True:
            self.name = name
        else:
            raise TypeError('name must be an instance of (str, type(None))')
        if isinstance(port_names, (list, type(None))) is True:
            self.port_names = port_names
        else:
            raise TypeError('port_names must be an instance of (list, type(None))')


class OneHundredGbe(object):
    """Generated from OpenAPI #/components/schemas/Layer1.OneHundredGbe model

    100 gigabit ethernet settings  

    Args
    ----
    - ieee_media_defaults (Union[True, False]): Enable/disable ieee media default settings. True will override the speed, auto_negotiate, link_training, rs_fec settings
    - auto_negotiate (Union[True, False]): Enable/disable auto negotiation
    - link_training (Union[True, False]): Enable/disable link training
    - rs_fec (Union[True, False]): Enable/disable reed solomon forward error correction (RS FEC)
    - speed (Union[one_hundred_gbps, fifty_gbps, forty_gbps, twenty_five_gpbs, ten_gbps]): This is the speed that will be used if auto_negotiate is false
    - flow_control (FlowControl): A container for layer1 receive flow control settings
    """
    def __init__(self, ieee_media_defaults=True, auto_negotiate=False, link_training=False, rs_fec=False, speed='one_hundred_gbps', flow_control=None):
        from abstract_open_traffic_generator.layer1 import FlowControl
        if isinstance(ieee_media_defaults, (bool, type(None))) is True:
            self.ieee_media_defaults = ieee_media_defaults
        else:
            raise TypeError('ieee_media_defaults must be an instance of (bool, type(None))')
        if isinstance(auto_negotiate, (bool, type(None))) is True:
            self.auto_negotiate = auto_negotiate
        else:
            raise TypeError('auto_negotiate must be an instance of (bool, type(None))')
        if isinstance(link_training, (bool, type(None))) is True:
            self.link_training = link_training
        else:
            raise TypeError('link_training must be an instance of (bool, type(None))')
        if isinstance(rs_fec, (bool, type(None))) is True:
            self.rs_fec = rs_fec
        else:
            raise TypeError('rs_fec must be an instance of (bool, type(None))')
        if isinstance(speed, (str, type(None))) is True:
            self.speed = speed
        else:
            raise TypeError('speed must be an instance of (str, type(None))')
        if isinstance(flow_control, (FlowControl, type(None))) is True:
            self.flow_control = flow_control
        else:
            raise TypeError('flow_control must be an instance of (FlowControl, type(None))')


class Ethernet(object):
    """Generated from OpenAPI #/components/schemas/Layer1.Ethernet model

    10/100/1000 Ethernet settings  

    Args
    ----
    - media (Union[copper, fiber]): TBD
    - speed (Union[one_thousand_mbps, one_hundred_fd_mbps, one_hundred_hd_mbps, ten_fd_mbps, ten_hd_mbps]): This is the speed that will be used if auto_negotiate is false
    - auto_negotiate (Union[True, False]): Enable/disable auto negotiation
    - advertise_one_thousand_mbps (Union[True, False]): If auto_negotiate is true then this speed will be advertised
    - advertise_one_hundred_fd_mbps (Union[True, False]): If auto_negotiate is true then this speed will be advertised
    - advertise_one_hundred_hd_mbps (Union[True, False]): If auto_negotiate is true then this speed will be advertised
    - advertise_ten_fd_mbps (Union[True, False]): If auto_negotiate is true then this speed will be advertised
    - advertise_ten_hd_mbps (Union[True, False]): If auto_negotiate is true then this speed will be advertised
    - flow_control (FlowControl): A container for layer1 receive flow control settings
    """
    def __init__(self, media='copper', speed='one_thousand_mbps', auto_negotiate=True, advertise_one_thousand_mbps=True, advertise_one_hundred_fd_mbps=True, advertise_one_hundred_hd_mbps=True, advertise_ten_fd_mbps=True, advertise_ten_hd_mbps=True, flow_control=None):
        from abstract_open_traffic_generator.layer1 import FlowControl
        if isinstance(media, (str, type(None))) is True:
            self.media = media
        else:
            raise TypeError('media must be an instance of (str, type(None))')
        if isinstance(speed, (str, type(None))) is True:
            self.speed = speed
        else:
            raise TypeError('speed must be an instance of (str, type(None))')
        if isinstance(auto_negotiate, (bool, type(None))) is True:
            self.auto_negotiate = auto_negotiate
        else:
            raise TypeError('auto_negotiate must be an instance of (bool, type(None))')
        if isinstance(advertise_one_thousand_mbps, (bool, type(None))) is True:
            self.advertise_one_thousand_mbps = advertise_one_thousand_mbps
        else:
            raise TypeError('advertise_one_thousand_mbps must be an instance of (bool, type(None))')
        if isinstance(advertise_one_hundred_fd_mbps, (bool, type(None))) is True:
            self.advertise_one_hundred_fd_mbps = advertise_one_hundred_fd_mbps
        else:
            raise TypeError('advertise_one_hundred_fd_mbps must be an instance of (bool, type(None))')
        if isinstance(advertise_one_hundred_hd_mbps, (bool, type(None))) is True:
            self.advertise_one_hundred_hd_mbps = advertise_one_hundred_hd_mbps
        else:
            raise TypeError('advertise_one_hundred_hd_mbps must be an instance of (bool, type(None))')
        if isinstance(advertise_ten_fd_mbps, (bool, type(None))) is True:
            self.advertise_ten_fd_mbps = advertise_ten_fd_mbps
        else:
            raise TypeError('advertise_ten_fd_mbps must be an instance of (bool, type(None))')
        if isinstance(advertise_ten_hd_mbps, (bool, type(None))) is True:
            self.advertise_ten_hd_mbps = advertise_ten_hd_mbps
        else:
            raise TypeError('advertise_ten_hd_mbps must be an instance of (bool, type(None))')
        if isinstance(flow_control, (FlowControl, type(None))) is True:
            self.flow_control = flow_control
        else:
            raise TypeError('flow_control must be an instance of (FlowControl, type(None))')


class FlowControl(object):
    """Generated from OpenAPI #/components/schemas/Layer1.FlowControl model

    A container for layer1 receive flow control settings  

    Args
    ----
    - directed_address (str): The 48bit mac address that the layer1 port names will listen on for a directed pause.
    - choice (Union[Ieee8021qbb, Ieee8023x]): The type of priority flow control
    """
    _CHOICE_MAP = {
        'Ieee8021qbb': 'ieee_802_1qbb',
        'Ieee8023x': 'ieee_802_3x',
    }
    def __init__(self, directed_address='0180C2000001', choice=None):
        from abstract_open_traffic_generator.layer1 import Ieee8021qbb
        from abstract_open_traffic_generator.layer1 import Ieee8023x
        if isinstance(choice, (Ieee8021qbb, Ieee8023x)) is False:
            raise TypeError('choice must be of type: Ieee8021qbb, Ieee8023x')
        self.__setattr__('choice', FlowControl._CHOICE_MAP[type(choice).__name__])
        self.__setattr__(FlowControl._CHOICE_MAP[type(choice).__name__], choice)
        if isinstance(directed_address, (str, type(None))) is True:
            self.directed_address = directed_address
        else:
            raise TypeError('directed_address must be an instance of (str, type(None))')


class Ieee8023x(object):
    """Generated from OpenAPI #/components/schemas/Layer1.Ieee8023x model

    A container for ieee 802.3x rx pause settings  
    """
    def __init__(self):
        pass


class Ieee8021qbb(object):
    """Generated from OpenAPI #/components/schemas/Layer1.Ieee8021qbb model

    These settings enhance the existing 802.3x pause priority capabilities to enable flow control based on 802.1p priorities (classes of service)  

    Args
    ----
    - pfc_delay (int): The upper limit on the transmit time of a queue after receiving a message to pause a specified priority. A value of 0 or null indicates that pfc delay will not be enabled.
    - pfc_class_0 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_1 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_2 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_3 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_4 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_5 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_6 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    - pfc_class_7 (int): The valid values are null, 0
        7. A null value indicates there is no setting for this pfc class
    """
    def __init__(self, pfc_delay=None, pfc_class_0=None, pfc_class_1=None, pfc_class_2=None, pfc_class_3=None, pfc_class_4=None, pfc_class_5=None, pfc_class_6=None, pfc_class_7=None):
        if isinstance(pfc_delay, (float, int, type(None))) is True:
            self.pfc_delay = pfc_delay
        else:
            raise TypeError('pfc_delay must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_0, (float, int, type(None))) is True:
            self.pfc_class_0 = pfc_class_0
        else:
            raise TypeError('pfc_class_0 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_1, (float, int, type(None))) is True:
            self.pfc_class_1 = pfc_class_1
        else:
            raise TypeError('pfc_class_1 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_2, (float, int, type(None))) is True:
            self.pfc_class_2 = pfc_class_2
        else:
            raise TypeError('pfc_class_2 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_3, (float, int, type(None))) is True:
            self.pfc_class_3 = pfc_class_3
        else:
            raise TypeError('pfc_class_3 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_4, (float, int, type(None))) is True:
            self.pfc_class_4 = pfc_class_4
        else:
            raise TypeError('pfc_class_4 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_5, (float, int, type(None))) is True:
            self.pfc_class_5 = pfc_class_5
        else:
            raise TypeError('pfc_class_5 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_6, (float, int, type(None))) is True:
            self.pfc_class_6 = pfc_class_6
        else:
            raise TypeError('pfc_class_6 must be an instance of (float, int, type(None))')
        if isinstance(pfc_class_7, (float, int, type(None))) is True:
            self.pfc_class_7 = pfc_class_7
        else:
            raise TypeError('pfc_class_7 must be an instance of (float, int, type(None))')





class Config(object):
    """Generated from OpenAPI #/components/schemas/Config.Config model

    A container for all models that are part of the configuration  

    Args
    ----
    - ports (list[Port]): The ports that will be configured on the traffic generator
    - device_groups (list[DeviceGroup]): The device groups that will be configured on the traffic generator
    - flows (list[Flow]): The flows that will be configured on the traffic generator
    - layer1 (list[Layer1]): The layer1 settings that will be configured on the traffic generator
    - captures (list[Capture]): The captures that will be configured on the traffic generator
    - options (Options): Global configuration options
    """
    def __init__(self, ports=[], device_groups=[], flows=[], layer1=[], captures=[], options=None):
        from abstract_open_traffic_generator.config import Options
        if isinstance(ports, (list, type(None))) is True:
            self.ports = ports
        else:
            raise TypeError('ports must be an instance of (list, type(None))')
        if isinstance(device_groups, (list, type(None))) is True:
            self.device_groups = device_groups
        else:
            raise TypeError('device_groups must be an instance of (list, type(None))')
        if isinstance(flows, (list, type(None))) is True:
            self.flows = flows
        else:
            raise TypeError('flows must be an instance of (list, type(None))')
        if isinstance(layer1, (list, type(None))) is True:
            self.layer1 = layer1
        else:
            raise TypeError('layer1 must be an instance of (list, type(None))')
        if isinstance(captures, (list, type(None))) is True:
            self.captures = captures
        else:
            raise TypeError('captures must be an instance of (list, type(None))')
        if isinstance(options, (Options, type(None))) is True:
            self.options = options
        else:
            raise TypeError('options must be an instance of (Options, type(None))')


class Options(object):
    """Generated from OpenAPI #/components/schemas/Config.Options model

    Global configuration options  

    Args
    ----
    - port_options (Options): Common port options that apply to all configured Port.Port objects.
    """
    def __init__(self, port_options=None):
        from abstract_open_traffic_generator.port import Options
        if isinstance(port_options, (Options, type(None))) is True:
            self.port_options = port_options
        else:
            raise TypeError('port_options must be an instance of (Options, type(None))')



import pytest
from abstract_open_traffic_generator.flow import *
from abstract_open_traffic_generator.flow_ipv4 import Priority, Dscp


def test_ipv4_fixed_priority_dscp(serializer):
    dscp = Dscp(phb=Pattern(Dscp.PHB_CS6))
    dscp_priority = Priority(dscp)
    print(serializer.json(dscp_priority))


def test_ipv4_list_priority_dscp(serializer):
    phb_list = [
        Dscp.PHB_AF11,
        Dscp.PHB_AF21,
        Dscp.PHB_AF43,
        Dscp.PHB_CS3,
        Dscp.PHB_CS7
    ]
    dscp = Dscp(phb=Pattern(phb_list))
    dscp_priority = Priority(dscp)
    print(serializer.json(dscp_priority))


def test_ipv4_counter_priority_dscp(serializer):
    phb_counter = Counter(start='10', step='1', count=6, up=True)
    dscp = Dscp(phb=Pattern(phb_counter))
    dscp_priority = Priority(dscp)
    print(serializer.json(dscp_priority))


def test_ipv4_priority_dscp_ecn(serializer):
    dscp = Dscp(phb=Pattern(Dscp.PHB_CS7), 
        ecn=Pattern(Dscp.ECN_CAPABLE_TRANSPORT_1))
    dscp_priority = Priority(dscp)
    print(serializer.json(dscp_priority))


def test_ipv4_priority_raw(serializer):
    counter = Counter(start='1', step='1', count=4, up=True)
    pattern = Pattern(counter)
    raw_priority = Priority(pattern)
    print(serializer.json(raw_priority))


def test_ipv4(serializer):
    ipv4 = Ipv4(src= Pattern(Counter(start='1.1.1.1', step='0.0.0.1', count=10)), 
        dst=Pattern(Counter(start='1.1.2.1', step='0.0.0.1', count=10)), 
        priority=Priority(Pattern(Priority.PRIORITY_RAW)))
    print(serializer.json(ipv4))


if __name__ == '__main__':
    pytest.main(['-s', __file__])



import os
import shutil
from setuptools import setup, find_namespace_packages


pkg_name = 'abstract_open_traffic_generator'
base_dir = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(base_dir, 'README.md')) as fid:
    long_description = fid.read()
with open(os.path.join(base_dir, 'VERSION')) as fid:
    version_number = fid.read()

shutil.rmtree(os.path.join(base_dir, pkg_name, 'tests'), ignore_errors=True)
shutil.rmtree(os.path.join(base_dir, pkg_name, 'docs'), ignore_errors=True)

shutil.copytree(os.path.join(base_dir, 'tests'), 
    os.path.join(base_dir, pkg_name, 'tests'))
os.mkdir(os.path.join(base_dir, pkg_name, 'docs'))
shutil.copy(os.path.join(base_dir, 'models', 'openapi.yaml'),
    os.path.join(base_dir, pkg_name, 'docs'))

setup(
    name=pkg_name,
    version=version_number,
    description='The Abstract Open Traffic Generator Python Package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/open-traffic-generator',
    author='ajbalogh',
    author_email='andy.balogh@keysight.com',
    license='MIT',
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing :: Traffic Generation',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3'
    ],
    keywords='abstract testing open traffic generator automation',
    packages=[pkg_name],
    include_package_data=True,
    python_requires='>=2.7, <4',
    tests_require=['pytest']
)






class Errors(object):
    """Generated from OpenAPI #/components/schemas/Result.Errors model

    TBD  

    Args
    ----
    - errors (list[Error]): TBD
    """
    def __init__(self, errors=[]):
        if isinstance(errors, (list, type(None))) is True:
            self.errors = errors
        else:
            raise TypeError('errors must be an instance of (list, type(None))')


class Error(object):
    """Generated from OpenAPI #/components/schemas/Result.Error model

    TBD  

    Args
    ----
    - name (str): The unique name of an object in the configuration
    - message (str): Detailed error information
    """
    def __init__(self, name=None, message=None):
        if isinstance(name, (str, type(None))) is True:
            self.name = name
        else:
            raise TypeError('name must be an instance of (str, type(None))')
        if isinstance(message, (str, type(None))) is True:
            self.message = message
        else:
            raise TypeError('message must be an instance of (str, type(None))')


class Capability(object):
    """Generated from OpenAPI #/components/schemas/Result.Capability model

    TBD  

    Args
    ----
    - unsupported (list[str]): A list of #/components/schemas/... path that are not supported
    - formats (list[str]): A #/components/schemas/... path and specific format details regarding the path. Specific model format details can be additional objects and properties represented as a hashmap. For example layer1 models are defined as a hashmap key to object with each object consisting of a specific name/value property pairs. This list of items will detail any specific formats, properties, enums
    """
    def __init__(self, unsupported=[], formats=[]):
        if isinstance(unsupported, (list, type(None))) is True:
            self.unsupported = unsupported
        else:
            raise TypeError('unsupported must be an instance of (list, type(None))')
        if isinstance(formats, (list, type(None))) is True:
            self.formats = formats
        else:
            raise TypeError('formats must be an instance of (list, type(None))')


class PortRequest(object):
    """Generated from OpenAPI #/components/schemas/Result.PortRequest model

    The flow result request to the traffic generator   

    Args
    ----
    - port_names (list[str]): The names of objects to return results for. An empty list will return all port row results
    - columns (list[Union[name, location, link, capture, frames_tx, frames_rx, frames_tx_rate, frames_rx_rate, bytes_tx, bytes_rx, bytes_tx_rate, bytes_rx_rate, pfc_class_0_frames_rx, pfc_class_1_frames_rx, pfc_class_2_frames_rx, pfc_class_3_frames_rx, pfc_class_4_frames_rx, pfc_class_5_frames_rx, pfc_class_6_frames_rx, pfc_class_7_frames_rx]]): The names of columns to return results for. An empty list will return all columns. The name column will always be included as it is the unique key. The following is a description of the columns:
        name The name of a configured port
        location The state of the connection to the test port location. The format should be the configured port location along with any custom connection state message.
        link The state of the test port link The string can be up, down or a custom error message.
        capture The state of the test port capture infrastructure. The string can be started, stopped or a custom error message.
        frames_tx The current total number of frames transmitted
        frames_rx The current total number of valid frames received
        bytes_tx: The current total number of bytes transmitted
        bytes_rx: The current total number of valid bytes received
        frames_tx_rate: The current rate of frames transmitted
        frames_rx_rate: The current rate of valid frames received
        bytes_tx_rate: The current rate of bytes transmitted
        bytes_rx_rate: The current rate of bytes received
        pfc_class_0_frames_rx: The current total number of pfc class 0 frames received
        pfc_class_1_frames_rx: The current total number of pfc class 1 frames received
        pfc_class_2_frames_rx: The current total number of pfc class 2 frames received
        pfc_class_3_frames_rx: The current total number of pfc class 3 frames received
        pfc_class_4_frames_rx: The current total number of pfc class 4 frames received
        pfc_class_5_frames_rx: The current total number of pfc class 5 frames received
        pfc_class_6_frames_rx: The current total number of pfc class 6 frames received
        pfc_class_7_frames_rx: The current total number of pfc class 7 frames received
    """
    def __init__(self, port_names=[], columns=[]):
        if isinstance(port_names, (list, type(None))) is True:
            self.port_names = port_names
        else:
            raise TypeError('port_names must be an instance of (list, type(None))')
        if isinstance(columns, (list, type(None))) is True:
            self.columns = columns
        else:
            raise TypeError('columns must be an instance of (list, type(None))')


class Port(object):
    """Generated from OpenAPI #/components/schemas/Result.Port model

    A table of port results  

    Args
    ----
    - columns (list[str]): The columns requested
    - rows (list[list[str]]): A table of result values. Each row in rows is ordered by the columns property
    """
    def __init__(self, columns=[], rows=[]):
        if isinstance(columns, (list, type(None))) is True:
            self.columns = columns
        else:
            raise TypeError('columns must be an instance of (list, type(None))')
        if isinstance(rows, (list, type(None))) is True:
            self.rows = rows
        else:
            raise TypeError('rows must be an instance of (list, type(None))')


class FlowRequest(object):
    """Generated from OpenAPI #/components/schemas/Result.FlowRequest model

    The request to the traffic generator for flow results  

    Args
    ----
    - flow_names (list[str]): The names of flow objects to return results for. An empty list will return results for all flows
    - columns (list[Union[name, port_tx, port_rx, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, loss]]): The names of columns to return results for. An empty list will return all columns. The name column will always be included as it is the unique key. The following is a description of the columns:
        name The name of a configured flow
        port_tx The name of a configured port
        port_rx The name of a configured port
        frames_tx The current total number of frames transmitted
        frames_rx The current total number of valid frames received
        bytes_tx The current total number of bytes transmitted
        bytes_rx The current total number of bytes received
        frames_tx_rate The current rate of frames transmitted
        frames_rx_rate The current rate of valid frames received
        loss The percentage of lost frames
    - ingress_result_names (list[str]): Add any configured Flow.Pattern.ingress_result_name values that are to be included in the results. If the name is not configured then it will be excluded from the Result.Flow.columns and Result.Flow.rows. The name in the Result.Flow.columns will be a combination of the ingress_result_name and any system assigned name
    """
    def __init__(self, flow_names=[], columns=[], ingress_result_names=[]):
        if isinstance(flow_names, (list, type(None))) is True:
            self.flow_names = flow_names
        else:
            raise TypeError('flow_names must be an instance of (list, type(None))')
        if isinstance(columns, (list, type(None))) is True:
            self.columns = columns
        else:
            raise TypeError('columns must be an instance of (list, type(None))')
        if isinstance(ingress_result_names, (list, type(None))) is True:
            self.ingress_result_names = ingress_result_names
        else:
            raise TypeError('ingress_result_names must be an instance of (list, type(None))')


class Flow(object):
    """Generated from OpenAPI #/components/schemas/Result.Flow model

    A table of flow results  

    Args
    ----
    - columns (list[str]): The names of columns in the Request.Flow.columns and requested
    - rows (list[list[str]]): A table of result values. Each row in rows is ordered by the columns property
    """
    def __init__(self, columns=[], rows=[]):
        if isinstance(columns, (list, type(None))) is True:
            self.columns = columns
        else:
            raise TypeError('columns must be an instance of (list, type(None))')
        if isinstance(rows, (list, type(None))) is True:
            self.rows = rows
        else:
            raise TypeError('rows must be an instance of (list, type(None))')
